@@ -1,32 +1,67 @@
-#!/usr/bin/env python2
-# vim:fileencoding=utf-8
+#!/usr/bin/env python
 # License: GPLv3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import absolute_import, division, print_function, unicode_literals
+
+import unittest
 from collections import defaultdict
 from io import BytesIO
-import unittest
-
-from lxml import etree
 
 from calibre.ebooks.metadata.book import ALL_METADATA_FIELDS
 from calibre.ebooks.metadata.opf2 import OPF
 from calibre.ebooks.metadata.opf3 import (
-    parse_prefixes, reserved_prefixes, expand_prefix, read_identifiers,
-    read_metadata, set_identifiers, XPath, set_application_id, read_title,
-    read_refines, set_title, read_title_sort, read_languages, set_languages,
-    read_authors, Author, set_authors, ensure_prefix, read_prefixes,
-    read_book_producers, set_book_producers, read_timestamp, set_timestamp,
-    read_pubdate, set_pubdate, CALIBRE_PREFIX, read_last_modified, read_comments,
-    set_comments, read_publisher, set_publisher, read_tags, set_tags, read_rating,
-    set_rating, read_series, set_series, read_user_metadata, set_user_metadata,
-    read_author_link_map, read_user_categories, set_author_link_map, set_user_categories,
-    apply_metadata, read_raster_cover, ensure_is_only_raster_cover
+    CALIBRE_PREFIX,
+    Author,
+    XPath,
+    apply_metadata,
+    ensure_is_only_raster_cover,
+    ensure_prefix,
+    expand_prefix,
+    parse_prefixes,
+    read_authors,
+    read_book_producers,
+    read_comments,
+    read_identifiers,
+    read_languages,
+    read_last_modified,
+    read_link_maps,
+    read_metadata,
+    read_prefixes,
+    read_pubdate,
+    read_publisher,
+    read_raster_cover,
+    read_rating,
+    read_refines,
+    read_series,
+    read_tags,
+    read_timestamp,
+    read_title,
+    read_title_sort,
+    read_user_categories,
+    read_user_metadata,
+    reserved_prefixes,
+    set_application_id,
+    set_authors,
+    set_book_producers,
+    set_comments,
+    set_identifiers,
+    set_languages,
+    set_link_maps,
+    set_pubdate,
+    set_publisher,
+    set_rating,
+    set_series,
+    set_tags,
+    set_timestamp,
+    set_title,
+    set_user_categories,
+    set_user_metadata,
 )
+
 # This import is needed to prevent a test from running slowly
 from calibre.ebooks.oeb.polish.pretty import pretty_opf, pretty_xml_tree  # noqa
+from calibre.utils.xml_parse import safe_xml_fromstring
 
-read_author_link_map, read_user_categories, set_author_link_map, set_user_categories
+read_user_categories, set_user_categories, read_link_maps, set_link_maps
 
 TEMPLATE = '''<package xmlns="http://www.idpf.org/2007/opf" version="3.0" prefix="calibre: %s" unique-identifier="uid"><metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">{metadata}</metadata><manifest>{manifest}</manifest></package>''' % CALIBRE_PREFIX  # noqa
 default_refines = defaultdict(list)
@@ -37,7 +72,7 @@ class TestOPF3(unittest.TestCase):
     ae = unittest.TestCase.assertEqual
 
     def get_opf(self, metadata='', manifest=''):
-        return etree.fromstring(TEMPLATE.format(metadata=metadata, manifest=manifest))
+        return safe_xml_fromstring(TEMPLATE.format(metadata=metadata, manifest=manifest))
 
     def test_prefix_parsing(self):  # {{{
         self.ae(parse_prefixes('foaf: http://xmlns.com/foaf/spec/\n dbp: http://dbpedia.org/ontology/'),
@@ -290,10 +325,10 @@ class TestOPF3(unittest.TestCase):
             f = globals()['set_' + name]
             f(root, read_prefixes(root), read_refines(root), val)
             return rt(root, name)
-        for name in 'author_link_map user_categories'.split():
+        for name in 'link_maps user_categories'.split():
             root = self.get_opf('''<meta name="calibre:%s" content='{"1":1}'/>''' % name)
             self.ae({'1':1}, rt(root, name))
-            root = self.get_opf('''<meta name="calibre:%s" content='{"1":1}'/><meta property="calibre:%s">{"2":2}</meta>''' % (name, name))
+            root = self.get_opf(f'''<meta name="calibre:{name}" content='{{"1":1}}'/><meta property="calibre:{name}">{{"2":2}}</meta>''')
             self.ae({'2':2}, rt(root, name))
             self.ae({'3':3}, st(root, name, {3:3}))
 
@@ -330,7 +365,7 @@ class TestOPF3(unittest.TestCase):
         <dc:subject>conversion</dc:subject>
         <dc:subject>docs</dc:subject>
         <dc:subject>ebook</dc:subject>
-        <meta content="{&quot;Kovid Goyal&quot;: &quot;&quot;}" name="calibre:author_link_map"/>
+        <meta content="{&quot;Kovid Goyal&quot;: &quot;https://kovidgoyal.net&quot;}" name="calibre:author_link_map"/>
         <meta content="Demos" name="calibre:series"/>
         <meta content="1" name="calibre:series_index"/>
         <meta content="10" name="calibre:rating"/>
@@ -520,10 +555,10 @@ class TestOPF3(unittest.TestCase):
             for field in ALL_METADATA_FIELDS:
                 if field not in 'manifest spine':
                     v2, v3 = getattr(mi2, field, None), getattr(mi3, field, None)
-                    self.ae(v2, v3, '%s: %r != %r' % (field, v2, v3))
+                    self.ae(v2, v3, f'{field}: {v2!r} != {v3!r}')
 
         mi2 = OPF(BytesIO(raw.encode('utf-8'))).to_book_metadata()
-        root = etree.fromstring(raw)
+        root = safe_xml_fromstring(raw)
         root.set('version', '3.0')
         mi3, _, raster_cover, first_spine_item  = read_metadata(root, return_extra_data=True)
         self.assertIsNone(raster_cover)
